@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -67,7 +67,7 @@ unsigned int pth_sleep(unsigned int sec)
     offset = pth_time(sec, 0);
     pth_time_set(&until, PTH_TIME_NOW);
     pth_time_add(&until, &offset);
-    
+
     /* and let thread sleep until this time is elapsed */
     ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, until);
     pth_wait(ev);
@@ -134,14 +134,14 @@ int pth_sigwait_ev(const sigset_t *set, int *sigp, pth_event_t ev_extra)
 }
 
 /* Pth variant of select(2) */
-int pth_select(int nfds, fd_set *readfds, fd_set *writefds, 
+int pth_select(int nfds, fd_set *readfds, fd_set *writefds,
                fd_set *exceptfds, struct timeval *timeout)
 {
     return pth_select_ev(nfds, readfds, writefds, exceptfds, timeout, NULL);
 }
 
 /* Pth variant of select(2) with extra events */
-int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds, 
+int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
                   fd_set *efds, struct timeval *timeout, pth_event_t ev_extra)
 {
     struct timeval delay;
@@ -154,7 +154,7 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
     fd_set *rtmp, *wtmp, *etmp;
     int selected;
     int rc;
-    
+
     pth_implicit_init();
     pth_debug2("pth_select_ev: called from thread \"%s\"", pth_current->name);
 
@@ -163,13 +163,13 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
         if (timeout->tv_sec < 0 || timeout->tv_usec < 0)
             return_errno(-1, EINVAL);
         if (timeout->tv_sec == 0 && timeout->tv_usec < 500000) {
-            /* very small delays we can accept to perform directly */
-            while (   pth_sc(select)(0, NULL, NULL, NULL, timeout) < 0 
+            /* very small delays are acceptable to be performed directly */
+            while (   pth_sc(select)(0, NULL, NULL, NULL, timeout) < 0
                    && errno == EINTR) ;
         }
         else {
             /* larger delays have to use the scheduler */
-            ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key_timeout, 
+            ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key_timeout,
                            pth_timeout(timeout->tv_sec, timeout->tv_usec));
             if (ev_extra != NULL)
                 pth_event_concat(ev, ev_extra, NULL);
@@ -187,12 +187,12 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
         return 0;
     }
 
-    /* now directly poll filedescriptor sets to avoid unneccessary 
+    /* now directly poll filedescriptor sets to avoid unneccessary
        (and resource consuming because of context switches, etc) event
        handling through the scheduler. We've to be carefully here, because not
        all platforms guarranty us that the sets are unmodified when an error
        or timeout occured. */
-    delay.tv_sec  = 0; 
+    delay.tv_sec  = 0;
     delay.tv_usec = 0;
     rtmp = NULL;
     if (rfds != NULL) {
@@ -209,7 +209,7 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
         espare = *efds;
         etmp = &espare;
     }
-    while ((rc = pth_sc(select)(nfd, rtmp, wtmp, etmp, &delay)) < 0 && errno == EINTR) ; 
+    while ((rc = pth_sc(select)(nfd, rtmp, wtmp, etmp, &delay)) < 0 && errno == EINTR) ;
     if (rc > 0) {
         if (rfds != NULL)
             *rfds = rspare;
@@ -231,11 +231,11 @@ int pth_select_ev(int nfd, fd_set *rfds, fd_set *wfds,
 
     /* suspend current thread until one fd is ready or the timeout occurred */
     rc = -1;
-    ev = ev_select = pth_event(PTH_EVENT_SELECT|PTH_MODE_STATIC, 
+    ev = ev_select = pth_event(PTH_EVENT_SELECT|PTH_MODE_STATIC,
                                &ev_key_select, &rc, nfd, rfds, wfds, efds);
     ev_timeout = NULL;
     if (timeout != NULL) {
-        ev_timeout = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key_timeout, 
+        ev_timeout = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key_timeout,
                                pth_timeout(timeout->tv_sec, timeout->tv_usec));
         pth_event_concat(ev, ev_timeout, NULL);
     }
@@ -268,7 +268,7 @@ int pth_poll(struct pollfd *pfd, nfds_t nfd, int timeout)
     return pth_poll_ev(pfd, nfd, timeout, NULL);
 }
 
-/* Pth variant of poll(2) with extra events: 
+/* Pth variant of poll(2) with extra events:
    NOTICE: THIS HAS TO BE BASED ON pth_select(2) BECAUSE
            INTERNALLY THE SCHEDULER IS ONLY select(2) BASED!! */
 int pth_poll_ev(struct pollfd *pfd, nfds_t nfd, int timeout, pth_event_t ev_extra)
@@ -302,7 +302,7 @@ int pth_poll_ev(struct pollfd *pfd, nfds_t nfd, int timeout, pth_event_t ev_extr
         ptv->tv_sec  = timeout / 1000;
         ptv->tv_usec = (timeout % 1000) * 1000;
     }
-    else 
+    else
         return_errno(-1, EINVAL);
 
     /* create fd sets and determine max fd */
@@ -379,11 +379,22 @@ int pth_connect_ev(int s, const struct sockaddr *addr, socklen_t addrlen, pth_ev
     static pth_key_t ev_key = PTH_KEY_INIT;
     int rv, err;
     socklen_t errlen;
-  
+    int fdmode;
+
+    pth_implicit_init();
+    pth_debug2("pth_connect_ev: enter from thread \"%s\"", pth_current->name);
+
+    /* force filedescriptor into non-blocking mode */
+    fdmode = pth_fdmode(s, PTH_FDMODE_NONBLOCK);
+
     /* try to connect */
-    while (   (rv = pth_sc(connect)(s, (struct sockaddr *)addr, addrlen)) == -1 
+    while (   (rv = pth_sc(connect)(s, (struct sockaddr *)addr, addrlen)) == -1
            && errno == EINTR)
         ;
+
+    /* restore filedescriptor mode */
+    errno_shield { pth_fdmode(s, fdmode); }
+
     /* when it is still on progress wait until socket is really writeable */
     if (rv == -1 && errno == EINPROGRESS) {
         ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, s);
@@ -398,10 +409,12 @@ int pth_connect_ev(int s, const struct sockaddr *addr, socklen_t addrlen, pth_ev
         errlen = sizeof(err);
         if (getsockopt(s, SOL_SOCKET, SO_ERROR, (void *)&err, &errlen) == -1)
             return -1;
-        if (err == 0) 
+        if (err == 0)
             return 0;
         return_errno(rv, err);
     }
+
+    pth_debug2("pth_connect_ev: leave to thread \"%s\"", pth_current->name);
     return rv;
 }
 
@@ -449,7 +462,7 @@ int pth_accept_ev(int s, struct sockaddr *addr, socklen_t *addrlen, pth_event_t
 
     /* restore filedescriptor mode */
     errno_shield {
-        pth_fdmode(s, fdmode); 
+        pth_fdmode(s, fdmode);
         if (rv != -1)
             pth_fdmode(rv, fdmode);
     }
@@ -483,17 +496,17 @@ ssize_t pth_read_ev(int fd, void *buf, size_t nbytes, pth_event_t ev_extra)
     /* poll filedescriptor when not already in non-blocking operation */
     if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
 
-        /* now directly poll filedescriptor for readability 
+        /* now directly poll filedescriptor for readability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
-        delay.tv_sec  = 0; 
+        delay.tv_sec  = 0;
         delay.tv_usec = 0;
-        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
-               && errno == EINTR) ; 
+        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0
+               && errno == EINTR) ;
 
-        /* when filedescriptor is still not readable, 
+        /* when filedescriptor is still not readable,
            let thread sleep until it is or the extra event occurs */
         if (n < 1) {
             ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
@@ -513,8 +526,8 @@ ssize_t pth_read_ev(int fd, void *buf, size_t nbytes, pth_event_t ev_extra)
        above by polling that the next read(2) call will not block.  But keep
        in mind, that only 1 next read(2) call is guarrantied to not block
        (except for the EINTR situation). */
-    while ((n = pth_sc(read)(fd, buf, nbytes)) < 0 
-           && errno == EINTR) ;                    
+    while ((n = pth_sc(read)(fd, buf, nbytes)) < 0
+           && errno == EINTR) ;
 
     pth_debug2("pth_read_ev: leave to thread \"%s\"", pth_current->name);
     return n;
@@ -551,19 +564,19 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     /* poll filedescriptor when not already in non-blocking operation */
     if (fdmode != PTH_FDMODE_NONBLOCK) {
 
-        /* now directly poll filedescriptor for writeability 
+        /* now directly poll filedescriptor for writeability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
-        delay.tv_sec  = 0; 
+        delay.tv_sec  = 0;
         delay.tv_usec = 0;
-        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
-               && errno == EINTR) ; 
+        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0
+               && errno == EINTR) ;
 
         rv = 0;
         for (;;) {
-            /* when filedescriptor is still not writeable, 
+            /* when filedescriptor is still not writeable,
                let thread sleep until it is or event occurs */
             if (n < 1) {
                 ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
@@ -580,7 +593,7 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
             }
 
             /* now perform the actual write operation */
-            while ((s = pth_sc(write)(fd, buf, nbytes)) < 0 
+            while ((s = pth_sc(write)(fd, buf, nbytes)) < 0
                    && errno == EINTR) ;
             if (s > 0)
                 rv += s;
@@ -605,7 +618,7 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
     }
     else {
         /* Now perform the actual write operation */
-        while ((rv = pth_sc(write)(fd, buf, nbytes)) < 0 
+        while ((rv = pth_sc(write)(fd, buf, nbytes)) < 0
                && errno == EINTR) ;
     }
 
@@ -622,24 +635,24 @@ pid_t pth_waitpid(pid_t wpid, int *status, int options)
     pth_event_t ev;
     static pth_key_t ev_key = PTH_KEY_INIT;
     pid_t pid;
-    
+
     pth_debug2("pth_waitpid: called from thread \"%s\"", pth_current->name);
-    
-    /* do a non-blocking poll for the pid */
-    while ((pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0 && errno == EINTR) ;
-    /* if pid was found or caller requested a polling return immediately */
-    if (pid < 0 || pid > 0)
-        return pid;
-    if (pid == 0 && (options & WNOHANG))
-        return 0;
 
-    /* else create a pid event and block current thread until it occurred */
-    ev = pth_event(PTH_EVENT_PID|PTH_MODE_STATIC, 
-                   &ev_key, wpid, status, options);
-    pth_wait(ev);
+    for (;;) {
+        /* do a non-blocking poll for the pid */
+        while (   (pid = pth_sc(waitpid)(wpid, status, options|WNOHANG)) < 0 
+               && errno == EINTR) ;
 
-    /* finally do the waitpid again for the result value and return it (?REALLY?) */
-    /* pid = pth_sc(waitpid)(wpid, status, options); */
+        /* if pid was found or caller requested a polling return immediately */
+        if (pid == -1 || pid > 0 || (pid == 0 && (options & WNOHANG)))
+            break;
+
+        /* else wait a little bit */
+        ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key, pth_timeout(0,250000));
+        pth_wait(ev);
+    }
+
+    pth_debug2("pth_waitpid: leave to thread \"%s\"", pth_current->name);
     return pid;
 }
 
@@ -668,17 +681,17 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
     /* poll filedescriptor when not already in non-blocking operation */
     if (pth_fdmode(fd, PTH_FDMODE_POLL) == PTH_FDMODE_BLOCK) {
 
-        /* first directly poll filedescriptor for readability 
+        /* first directly poll filedescriptor for readability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
-        delay.tv_sec  = 0; 
+        delay.tv_sec  = 0;
         delay.tv_usec = 0;
-        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
-               && errno == EINTR) ; 
+        while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0
+               && errno == EINTR) ;
 
-        /* when filedescriptor is still not readable, 
+        /* when filedescriptor is still not readable,
            let thread sleep until it is or event occurs */
         if (n < 1) {
             ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
@@ -699,10 +712,10 @@ ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev
        in mind, that only 1 next read(2) call is guarrantied to not block
        (except for the EINTR situation). */
 #if PTH_FAKE_RWV
-    while ((n = pth_readv_faked(fd, iov, iovcnt)) < 0 
+    while ((n = pth_readv_faked(fd, iov, iovcnt)) < 0
            && errno == EINTR) ;
 #else
-    while ((n = pth_sc(readv)(fd, iov, iovcnt)) < 0 
+    while ((n = pth_sc(readv)(fd, iov, iovcnt)) < 0
            && errno == EINTR) ;
 #endif
 
@@ -730,7 +743,7 @@ intern ssize_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
     /* allocate a temporary buffer */
     if ((buffer = (char *)malloc(bytes)) == NULL)
         return (ssize_t)(-1);
-    
+
     /* read data into temporary buffer (caller guarrantied us to not block) */
     rv = pth_sc(read)(fd, buffer, bytes);
 
@@ -747,7 +760,7 @@ intern ssize_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
         }
     }
 
-    /* remove the temporary buffer */ 
+    /* remove the temporary buffer */
     errno_shield { free(buffer); }
 
     /* return number of read bytes */
@@ -781,7 +794,7 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
     /* POSIX compliance */
     if (iovcnt <= 0 || iovcnt > UIO_MAXIOV)
         return_errno(-1, EINVAL);
-    
+
     /* force filedescriptor into non-blocking mode */
     fdmode = pth_fdmode(fd, PTH_FDMODE_NONBLOCK);
 
@@ -797,18 +810,18 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
         liovcnt = 0;
         pth_writev_iov_advance(iov, iovcnt, 0, &liov, &liovcnt);
 
-        /* first directly poll filedescriptor for writeability 
+        /* first directly poll filedescriptor for writeability
            to avoid unneccessary (and resource consuming because of context
            switches, etc) event handling through the scheduler */
         FD_ZERO(&fds);
         FD_SET(fd, &fds);
-        delay.tv_sec  = 0; 
+        delay.tv_sec  = 0;
         delay.tv_usec = 0;
-        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
-               && errno == EINTR) ; 
+        while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0
+               && errno == EINTR) ;
 
         for (;;) {
-            /* when filedescriptor is still not writeable, 
+            /* when filedescriptor is still not writeable,
                let thread sleep until it is or event occurs */
             if (n < 1) {
                 ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
@@ -826,10 +839,10 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
 
             /* Now perform the actual write operation */
 #if PTH_FAKE_RWV
-            while ((s = pth_writev_faked(fd, liov, liovcnt)) < 0 
+            while ((s = pth_writev_faked(fd, liov, liovcnt)) < 0
                    && errno == EINTR) ;
 #else
-            while ((s = pth_sc(writev)(fd, liov, liovcnt)) < 0 
+            while ((s = pth_sc(writev)(fd, liov, liovcnt)) < 0
                    && errno == EINTR) ;
 #endif
             if (s > 0)
@@ -856,10 +869,10 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
     else {
         /* Now perform the actual write operation */
 #if PTH_FAKE_RWV
-        while ((rv = pth_writev_faked(fd, iov, iovcnt)) < 0 
+        while ((rv = pth_writev_faked(fd, iov, iovcnt)) < 0
                && errno == EINTR) ;
 #else
-        while ((rv = pth_sc(writev)(fd, iov, iovcnt)) < 0 
+        while ((rv = pth_sc(writev)(fd, iov, iovcnt)) < 0
                && errno == EINTR) ;
 #endif
     }
@@ -959,7 +972,7 @@ intern ssize_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
     /* write continuous chunck of data (caller guarrantied us to not block) */
     rv = pth_sc(write)(fd, buffer, bytes);
 
-    /* remove the temporary buffer */ 
+    /* remove the temporary buffer */
     errno_shield { free(buffer); }
 
     return(rv);
@@ -989,7 +1002,7 @@ ssize_t pth_pread(int fd, void *buf, size_t nbytes, off_t offset)
 
     /* perform the read operation */
     rc = pth_read(fd, buf, nbytes);
-    
+
     /* restore the old offset situation */
     errno_shield { lseek(fd, old_offset, SEEK_SET); }
 
@@ -1022,9 +1035,9 @@ ssize_t pth_pwrite(int fd, const void *buf, size_t nbytes, off_t offset)
 
     /* perform the write operation */
     rc = pth_write(fd, buf, nbytes);
-    
+
     /* restore the old offset situation */
-    errno_shield { lseek(fd, old_offset, SEEK_SET); } 
+    errno_shield { lseek(fd, old_offset, SEEK_SET); }
 
     /* unprotect and return result of write */
     pth_mutex_release(&mutex);
