@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -22,6 +22,9 @@
 **
 **  test_httpd.c: Pth test program (faked HTTP daemon)
 */
+                             /* ``Unix is simple. It just takes a
+                                  genius to understand its simplicity.''
+                                            --- Dennis Ritchie           */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,12 +76,12 @@ static void *handler(void *_arg)
 
     /* generate response */
     sprintf(str, "HTTP/1.0 200 Ok\r\n"
-                 "Server: test_httpd/%x\r\n" 
+                 "Server: test_httpd/%x\r\n"
                  "Connection: close\r\n"
                  "Content-type: text/plain\r\n"
                  "\r\n"
                  "Just a trivial test for GNU Pth\n"
-                 "to show that it's serving data.\r\n", PTH_VERSION); 
+                 "to show that it's serving data.\r\n", PTH_VERSION);
     pth_write(fd, str, strlen(str));
 
     /* close connection and let thread die */
@@ -103,10 +106,10 @@ static void *ticker(void *_arg)
         ct = ctime(&now);
         ct[strlen(ct)-1] = NUL;
         pth_ctrl(PTH_CTRL_GETAVLOAD, &avload);
-        fprintf(stderr, "ticker woken up on %s, average load: %.2f\n", 
+        fprintf(stderr, "ticker woken up on %s, average load: %.2f\n",
                 ct, avload);
     }
-    /* NOT REACHED */
+    /* NOTREACHED */
     return NULL;
 }
 
@@ -132,7 +135,7 @@ static void myexit(int sig)
     exit(0);
 }
 
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
     struct sockaddr_in sar;
     struct protoent *pe;
@@ -216,7 +219,7 @@ int main(int argc, char *argv[])
             fprintf(stderr, "currently no more connections acceptable\n");
             continue;
         }
-        fprintf(stderr, "connection established (fd: %d, ip: %s, port: %d)\n", 
+        fprintf(stderr, "connection established (fd: %d, ip: %s, port: %d)\n",
                 sr, inet_ntoa(peer_addr.sin_addr), ntohs(peer_addr.sin_port));
 
         /* spawn new handling thread for connection */
